'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _createResourcesFromReducers2 = require('./createResourcesFromReducers');

var _createResourcesFromReducers3 = _interopRequireDefault(_createResourcesFromReducers2);

var _dispatch2 = require('./dispatch');

var _dispatch3 = _interopRequireDefault(_dispatch2);

var _getConsensus = require('./getConsensus');

var _getConsensus2 = _interopRequireDefault(_getConsensus);

var _callAction = require('./callAction');

var _callAction2 = _interopRequireDefault(_callAction);

var _types = require('./types');

var _connectActionSets2 = require('./connectActionSets');

var _connectActionSets3 = _interopRequireDefault(_connectActionSets2);

var Flambeau = (function () {
  /**
   * Creates a new Flambeau store
   */

  function Flambeau() {
    _classCallCheck(this, Flambeau);

    this.listeners = [];
  }

  /**
   * Attach reducers, with ids as keys, and reducer function sets as values.
   *
   * @param  {Object} idToReducers An object mapping identifiers to reducer function sets
   * @param  {Object} idToProps    (Optional) the unique attributes for each particular reducer
   */

  _createClass(Flambeau, [{
    key: 'setReducers',
    value: function setReducers(idToReducers) {
      var idToProps = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var _createResourcesFromReducers = (0, _createResourcesFromReducers3['default'])({
        reducers: idToReducers,
        idToProps: idToProps
      });

      var resources = _createResourcesFromReducers.resources;
      var states = _createResourcesFromReducers.states;

      this.resources = resources;
      this.states = states;
    }

    /**
     * Get multiple actions sets, connected to this store ready to dispatch to the reducers.
     *
     * @param  {Object} actionSets An object mapping action set identifiers to action functions
     * @return {Object}            The action set identifiers to connected action function sets.
     */
  }, {
    key: 'connectActionSets',
    value: function connectActionSets(actionSets) {
      var _this = this;

      return (0, _connectActionSets3['default'])({
        actionSets: actionSets,
        dispatch: function dispatch(action) {
          var changesStates = (0, _dispatch3['default'])({
            resources: _this.resources,
            states: _this.states
          })(action);

          _this.states = Object.assign({}, _this.states, changesStates);

          _this.listeners.forEach(function (callback) {
            callback(changesStates);
          });
        },
        getConsensusForActionSet: (0, _getConsensus2['default'])({
          resources: this.resources,
          getStates: function getStates() {
            return _this.states;
          }
        })
      });
    }

    /**
     * Get the state for a particular reducer.
     *
     * @param  {String} id The unique identifier for a particular reducer
     * @return {Any}    The state value of the reducer at the current time
     */
  }, {
    key: 'get',
    value: function get(id) {
      if (typeof id === 'undefined') {
        return this.states;
      } else if (typeof id === 'string') {
        return this.states[id];
      }
    }

    /**
     * Subscribe to reducer updates.
     *
     * @param  {Function} callback A function accepting an array of changed reducer identifiers
     * @return {Function}          A function used to unsubscribe when finished
     */
  }, {
    key: 'subscribe',
    value: function subscribe(callback) {
      var _this2 = this;

      this.listeners.push(callback);

      return function () {
        _this2.listeners.splice(_this2.listeners.indexOf(callback), 1);
      };
    }
  }]);

  return Flambeau;
})();

exports['default'] = Flambeau;
module.exports = exports['default'];