'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports['default'] = combineReducers;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _types = require('./types');

/**
 * Combines several Flambeau reducers into one, taking care of introspection
 *
 * @param  {Object} reducers                The reducers to be combined together, using the keys to inform the state’s structure
 * @param  {-> Object} getPropsByID(props)  A function returning the props for each individual reducer. It is passed the combined reducer’s props.
 * @return {Flambeau Reducer}               A new reducer combining those that were passed
 */

function combineReducers(reducers) {
	var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

	var _ref$getPropsByID = _ref.getPropsByID;
	var getPropsByID = _ref$getPropsByID === undefined ? function () {
		return {};
	} : _ref$getPropsByID;
	var alsoAdd = _ref.alsoAdd;

	var alsoResponder = undefined;
	var reducerIDs = Object.keys(reducers);

	var combinedReducer = _defineProperty({}, _types.GET_INITIAL_STATE, function (masterProps) {
		var propsByID = getPropsByID(masterProps);
		return reducerIDs.reduce(function (state, reducerID) {
			var reducer = reducers[reducerID];
			if (reducer[_types.GET_INITIAL_STATE]) {
				state[reducerID] = reducer[_types.GET_INITIAL_STATE](propsByID[reducerID]);
			}
			return state;
		}, {});
	});

	var actionSetHandlers = reducerIDs.reduce(function (actionSetHandlers, reducerID) {
		var reducer = reducers[reducerID];
		Object.keys(reducer).reduce(function (actionSetHandlers, property) {
			if (property === _types.GET_INITIAL_STATE || !!actionSetHandlers[property]) {
				return actionSetHandlers;
			}

			actionSetHandlers[property] = function (initialState, _ref2) {
				var isAction = _ref2.isAction;
				var isIntrospection = _ref2.isIntrospection;
				var props = _ref2.props;
				var forwardTo = _ref2.forwardTo;

				var propsByID = getPropsByID(props);
				if (isAction) {
					var newState = reducerIDs.reduce(function (state, reducerID) {
						state[reducerID] = forwardTo({
							responder: reducers[reducerID],
							initialState: initialState[reducerID],
							props: propsByID[reducerID]
						});
						return state;
					}, {});

					if (alsoResponder) {
						newState = forwardTo({
							responder: alsoResponder,
							initialState: newState,
							props: {
								props: props,
								propsByID: propsByID
							},
							sourceResponder: combinedReducer
						});
					}

					return newState;
				} else if (isIntrospection) {
					var _ret = (function () {
						if (alsoResponder) {
							// Use customized response, if one was given.
							var _response = forwardTo({
								responder: alsoResponder,
								initialState: initialState,
								props: {
									props: props,
									propsByID: propsByID
								},
								sourceResponder: combinedReducer
							});

							if (_response) {
								return {
									v: _response
								};
							}
						}

						// Use first reducer that responds.
						var response = undefined;
						reducerIDs.some(function (reducerID) {
							response = forwardTo({
								responder: reducers[reducerID],
								initialState: initialState[reducerID],
								props: propsByID[reducerID]
							});

							return typeof response !== 'undefined';
						});
						return {
							v: response
						};
					})();

					if (typeof _ret === 'object') return _ret.v;
				}
			};

			return actionSetHandlers;
		}, actionSetHandlers);

		return actionSetHandlers;
	}, {});

	Object.assign(combinedReducer, actionSetHandlers);

	if (alsoAdd) {
		alsoResponder = alsoAdd(combinedReducer);
	}

	return combinedReducer;
}

module.exports = exports['default'];